classdef EZStepper < hgsetget
    properties (SetAccess = private)
        PortObj = NaN; % COM port object; must be open
        ControllerType = 'EZHR17'; %supported: EZHR17
        ControllerName = NaN; %name or number of controller on this port
    end
    properties
        Timeout = 30; %Timeout for motor moves in sec
        LimitPolarity = 0; %0-NORMAL(ACTIVE HIGH), 1-INVERTED (ACTIVE LOW)
        LimitEnable = 1; %Enable limits
        HoldCurrent = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
        RunCurrent = 25; %current when moving; in % of full (2A for EZHR17); typically 25%
        StepDivide = 256; %number of microsteps per step; typically 256
        SlewVelocity = 100000; %Slew (max) velocity in microsteps/sec
        InitVelocity = 10000; %Initial velocity in microsteps/sec
        Ramp = 1000; %Acceleration (deceleration) in microsteps/sec^2
        Log = 0; %0 - do not accumulate log; 1 - accumulate log
        LastErr = 0; %Last error
    end
    properties (SetAccess = private, GetAccess = private) 
        nBits = 0; %Mode bits
    end
    methods
        function m = EZStepper(varargin)
            slewvelocity = m.SlewVelocity;
            initvelocity = m.InitVelocity;
            ramp = m.Ramp;
            stepdivide = m.StepDivide;
            holdcurrent = m.HoldCurrent; 
            runrurrent = m.RunCurrent;
            limitpolarity = m.LimitPolarity;
            limitenable = m.LimitEnable;
            
            property_argin = varargin;
            while (length(property_argin) >= 2)
                prop = property_argin{1};
                val = property_argin{2};
                property_argin = property_argin(3:end);
                switch prop
                case 'PortObj'
                    m.PortObj = val;
                case 'ControllerType'
                    m.ControllerType = val;
                case 'ControllerName'
                    m.ControllerName = val;
                case 'Timeout'
                    m.Timeout = val;
                case 'LimitPolarity'
                    limitpolarity = val; 
                case 'LimitEnable'
                    limitenable = val; 
                case 'HoldCurrent'
                    holdcurrent = val; 
                case 'RunCurrent'
                    runrurrent = val; 
                case 'StepDivide'
                    stepdivide = val; 
                case 'SlewVelocity'
                    slewvelocity = val; 
                case 'InitVelocity'
                    initvelocity = val; 
                case 'Ramp'
                    ramp = val; 
                case 'Log'
                    m.Log = val;
                otherwise
                    error([prop,' Is not a valid name; EZStepper properties: PortObj, ControllerType, ControllerName, Timeout, SlewVelocity, InitVelocity, Ramp, StepDivide, HoldCurrent, RunCurrent, LimitPolarity, LimitEnable, Log'])
                end
            end
            %LOAD PARAMETERS INTO CONTROLLER HERE
            %When parsing parameters must use local variables, not object properties 
            %because assignment to the properties will cause loading of parameters into controller
            %This loading can be done only later after all parameters are
            %parsed, (specifically because of log) and after the '&'
            %command is ussued and returned without error.
            
            %[~,e0] = MotorCmndRspn(m, '&', 1.0);
            [~,e0] = MotorCmndRspn(m, '&');
            if(e0==4), m.LastErr = 256; return; end
            m.SlewVelocity = slewvelocity; e1 = m.LastErr;
            m.InitVelocity = initvelocity; e2 = m.LastErr;
            m.Ramp = ramp; e3 = m.LastErr;
            m.StepDivide = stepdivide; e4 = m.LastErr;
            m.HoldCurrent = holdcurrent; e5 = m.LastErr;
            m.RunCurrent = runrurrent; e6 = m.LastErr;
            m.LimitPolarity = limitpolarity; e7 = m.LastErr; 
            m.LimitEnable = limitenable; e8 = m.LastErr;
            m.LastErr = e1 + e2*2 + e3*4 + e4*8 + e5*16 + e6*32 + e7*64 + e8*128;
        end
        function m = set.SlewVelocity(m,val)
           m.SlewVelocity = val; 
           MotorSetSlewVelo(m);
        end
        function m = set.InitVelocity(m,val)
           m.InitVelocity = val; 
           MotorSetInitVelo(m);
        end
        function m = set.Ramp(m,val)
           m.Ramp = val; 
           MotorSetRamp(m);
        end
        function m = set.StepDivide(m,val)
           m.StepDivide = val; 
           MotorSetDivide(m);
        end
        function m = set.HoldCurrent(m,val)
           m.HoldCurrent = val; 
           MotorSetHoldCurrent(m);
        end
        function m = set.RunCurrent(m,val)
           m.RunCurrent = val; 
           MotorSetRunCurrent(m);
        end
        function m = set.LimitPolarity(m,val)
           m.LimitPolarity = val; 
           MotorSetLimPolar(m);
        end
        function m = set.LimitEnable(m,val)
           m.LimitEnable = val; 
           MotorSetLimits(m);
        end
        function [response,error] = MotorCmndRspn(m, command, mode)
            if(nargin==3), readmode = mode; else readmode = 0; end
            %SENDS command STRING TO EZStepper CONTROLLER AND RETURNS response STRING
            %m is an object of EZStepper class
            global EZStepperLog
            extra = 3;
            crlf = sprintf('\n');
            serialobj = m.PortObj;
            mnum = m.ControllerName;
            cmd = ['/' int2str(mnum) command]; %FORM COMMAND
            FlushSerial(serialobj);
            fprintf(serialobj,cmd); %SEND COMMAND
            %res = fgets(serialobj); %READ RESPONSE (AND CLEAR THE BUFFER)
            if(readmode==0)
                [res, err] = ReadSerialToCRLF(serialobj,5); % Read to CR LF
            else
                res = ReadSerialToTimeout(serialobj,readmode); % use 3-rd parameter as timeout in sec.
                err = 0;
            end
            if(err>0)
                response = 'Error';
                error = 6; %Timeout
            else
                %disp([res ' ' num2str(numel(res))]);
                k = strfind(res,'/0'); %A VALID RESPONSE MUST CONTAIN THIS SUBSTRING ; 
                if(isempty(k))
                    response = 'Error';
                    error = 4; %Response does not contain /0
                else
                    ind = k(1);
                    if((length(res)-extra)<(ind+2))
                        response = 'Error';
                        error = 5; %Incomplete response
                    else
                        response = res(ind+2:length(res)-extra); %GET RID OF EXTRA CHARACTERS 
                        status = uint8(response(1:1)); %GET STATUS BYTE
                        error = double(bitand(status,15)); 
                        %0-No Error, 1-Initialization error, 2-Bad Command,
                        %3-Operand out of range (from previous command)
                    end
                end
            end
            set(m,'LastErr',error);
            if(m.Log == 1), EZStepperLog = [EZStepperLog cmd '-' res]; end
            if(error ~= 0), EZStepperLog = [EZStepperLog 'Error ' num2str(error) ': ' cmd '-' res crlf]; end
        end
        function [position,error] = MotorReadPosition(m)
            %RETURNS POSITION OF EZStepper CONTROLLER.
            %m is an object of EZStepper class
            [res,error] = MotorCmndRspn(m, '?0'); %SEND COMMAND TO INTERROGATE POSITION
            spos = res(2:length(res)); %GET RID OF THE STATUS BYTE
            if(error == 0)
                position = str2double(spos);
            else
                position = NaN;
            end
            if(isnan(position))
                error = 1;
            else
                error = 0;
            end
        end
        function [enc_pos,error] = MotorReadEncoder(m)
            %RETURNS POSITION OF EZStepper ENCODER.
            %m is an object of EZStepper class
            [res,error] = MotorCmndRspn(m, '?8'); %SEND COMMAND TO INTERROGATE ENCODER
            spos = res(2:length(res)); %GET RID OF THE STATUS BYTE
            if(error == 0)
                enc_pos = str2double(spos);
            else
                enc_pos = NaN;
            end
            if(isnan(enc_pos))
                error = 1;
            else
                error = 0;
            end
        end
        function error = MotorMoveRel(m,travel)
            %RELATIVE MOVE FOR EZStepper CONTROLLER
            %m is an object of EZStepper class
            if (travel==0) 
                error = 0; return;
            end
            %ROUND TRAVEL PARAMETER
            ratrav = round(abs(travel));
            cmnd = [sprintf('%d',ratrav) 'R'];
            if (travel > 0)
                cmnd = ['P' cmnd];
            else
                cmnd = ['D' cmnd];
            end
            [~,error] = MotorCmndRspn(m,cmnd);
        end
        function error = MotorMoveAbs(m,destination)
            %ABSOLUTE MOVE FOR EZHR17EN CONTROLLER.
            %m is an object of EZStepper class
            %ROUND DESTINATION PARAMETER
            rdest = round(destination);
            cmnd = ['A' sprintf('%d',rdest) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
        end
        function error = MotorWait(m)
            %WAITS FOR COMPLETION OF MOTION ON EZStepper CONTROLLER.
            %m is an object of EZStepper class
            tic;
            while 1
                [moving,error] = MotorMoveStatus(m);
                if((moving==0)&&(error==0))
                    break;
                end
                if(toc > m.Timeout)
                    error = -1; break;
                end
            end
        end
        function [moving,error] = MotorMoveStatus(m)
            %RETURNS MOVING STATUS OF EZStepper CONTROLLER
            %m is an object of EZStepper class
            %moving = 0 IS NOT MOVING; moving = 1 IS MOVING.
            [res,error] = MotorCmndRspn(m, 'Q');
            if(error>0)
                moving = 1;
            else
                status = uint8(res(1:1));
                ready = bitand(status,32);
                if(ready > 0)
                    moving = 0;
                else
                    moving = 1;
                end
            end
        end
        function error = MotorJog(m,speed)
            %JOG AT GIVEN SPEED FOR EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %SIGN OF speed DETERMINES DIRECTION. 
            %TO CHANGE DIRECTION CALL WITH ZERO SPEED FIRST.
            %ROUND SPEED PARAMETER
            rspd = round(speed);
            if (rspd == 0) 
                cmnd = 'T';
            end
            if (rspd > 0)
                cmnd = ['V' sprintf('%d',rspd) 'P0R'];
            end
            if (rspd < 0) 
                cmnd = ['V' sprintf('%d',-rspd) 'D0R'];
            end
            [~,error] = MotorCmndRspn(m,cmnd);
        end
        function error = MotorHome(m,speed)
            %HOME COMMAND OF EZStepper CONTROLLER
            %m is an object of EZStepper class
            %START MOVING HOME AT GIVEN SPEED; DO NOT WAIT FOR COMPLETION
            %DURING HOMING MOTOR MOVES IN THE DIRECTION OF DECREASING POSITION
            spd = fix(abs(speed));
            if(spd==0)
                %error('MotorHome: zero home speed');
                error = 1; 
                return;
            end
            %SET SPEED FOR HOMING
            set(m,'SlewVelocity',spd); %MotorSetSlewVelo(m,spd);
            %DO HOMING
            [~,error] = MotorCmndRspn(m,'Z1000000000R');
        end
        function error = MotorHomeComplete(m,speed)
            %HOME AND WAIT FOR COMPLETION WITH EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %ORIGINAL SLEW VELOCITY IS RESTORED AFTER HOMING.
            %DURING HOMING MOTOR MOVES IN THE DIRECTION OF DECREASING POSITION.

            %READ CURRENT SLEW VELOCITY
            [slewvelo,err1] = MotorReadSlewVelo(m);
            %DON'T DO HOMING IF READING VELOCITY RETURNED ERROR
            if(err1~=0), error = 1; return; end
            
            %START HOMING
            errt = 0;
            err2 = MotorHome(m,speed);
            if(err2~=0), errt = 2; end
            %WAIT FOR COMPLETION
            err3 = MotorWait(m);
            if(err3~=0), errt = errt+4; end
            %RESTORE SLEW VELOCITY
            set(m,'SlewVelocity',slewvelo); err4=0; %err4 = MotorSetSlewVelo(m,slewvelo);
            if(err4~=0), errt = errt+8; end
            error = errt;
        end
        function error = MotorOutputs(m,out)
            %On/Off Power Driver FOR EZStepper CONTROLLER
            %m is an object of EZStepper class
            %Interpret out as 2 bit Binary Value: 
            %3 - Both Drivers On; 2 - Driver2 On, Driver1 Off;
            %1 - Driver1 On, Driver2 Off; 0 - Both Drivers Off;
            switch out
                case {0, 1, 2, 3}
                    cmnd = ['J' sprintf('%d',out) 'R'];
                    [~,error] = MotorCmndRspn(m,cmnd);
                otherwise
                    error = -1;
            end
        end
        function [hardwr,error] = MotorReadHardwr(m)
            %READ CURRENT STATE OF SWITCHES FOR EZStepper CONTROLLER.
            %ONE or TWO CHARACTERS AFTER STATUS ENCODE SWITCHES AND OPTOS 
            %Bit 0 = Switch 1, Bit 1 = Switch 2, Bit 2 = Opto 1, Bit 3 = Opto 2
            %m is an object of EZStepper class
            cmnd = '?4';
            [res,error] = MotorCmndRspn(m,cmnd);
            if(error ~= 0), hardwr = NaN; return; end
            if(length(res)>1)
                n1 = str2double(res(2:2)); 
                hardwr = n1;
            end
            if(length(res)>2)
                n2 = str2double(res(3:3)); 
                hardwr = bitand(n1*10+n2,15);
            end
        end
        function [limits,error] = MotorReadLimits(m)
            %READ CURRENT STATE OF (OPTICAL) LIMIT SWITCHES FOR EZStepper CONTROLLER.
            %limits=0 if no limit switches are on,
            %      =1 if limit 1 is on,
            %      =2 if limit 2 is on,
            %      =3 if both limits are on.
            %m is an object of EZStepper class
            [hardwr,error] = MotorReadHardwr(m);
            if(error ~= 0), limits = NaN; return; end
            limits = fix(hardwr/4);
            if(m.LimitPolarity == 1), limits = bitxor(uint8(limits),uint8(3)); end
        end         
        function [slewvelo,error] = MotorReadSlewVelo(m)
            %READ SLEW VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            %m is an object of EZStepper class
            [res,error] = MotorCmndRspn(m,'?2');
            if(error ~= 0), slewvelo = NaN; return; end
            spos = res(2:length(res)); %GET RID OF THE STATUS BYTE
            slewvelo = str2double(spos); %IF NOT A REPRESENTATION OF A NUMBER - NaN
            if(isnan(slewvelo))
                error = 1;
            else
                error = 0;
            end
        end
%         function error = MotorSetOrigin(m,origin)
%             %SET ORIGIN FOR EZStepper CONTROLLER.
%             %OPTIONAL origin DEFINES SET POSITION COUNT
%             %m is an object of EZStepper class
%             if(nargin>1)
%                 org = fix(origin);
%             else
%                 org = 0;
%             end
%             cmnd = ['z' sprintf('%d',org) 'R'];
%             [~,error] = MotorCmndRspn(m,cmnd);
%             %SETTING ORIGIN MAY MOVE THE MOTOR TO THE NEXT FULL STEP, SO IT TAKES TIME
%             pause(0.5);
%         end
        function error = MotorSetOrigin(m,origin)
            %SET ORIGIN FOR EZStepper CONTROLLER.
            %OPTIONAL origin DEFINES SET POSITION COUNT
            %m is an object of EZStepper class
            if(nargin>1)
                org = fix(origin);
            else
                org = 0;
            end
            % Read current position
            [pos,errp] = MotorReadPosition(m);
            % Move half step for rounding. Must use "A" command.
            erra = MotorMoveAbs(m, pos+fix(m.StepDivide/2));
            errw = MotorWait(m);
            % Set position
            cmnd = ['z' sprintf('%d',org) 'R'];
            [~,errz] = MotorCmndRspn(m,cmnd);
            %SETTING ORIGIN MAY MOVE THE MOTOR TO THE FULL STEP, SO IT TAKES TIME
            pause(0.5);
            error = errz + 10*errw + 100*erra + 1000*errp;
        end
%         function [limits,error] = MotorToLimit(m,speed)
%             %MOVE UNTIL ANY LIMIT TURNS ON. RETURNS THE LIMIT THAT WAS HIT.
%             %IF ALREADY AT THE LIMIT, RETURNS IMMEDIATELY
%             %SIGN OF speed DETERMINES DIRECTION OF MOVE.
%             %limit: 1-limit 1(A), 2-limit 2(B)
%             %ORIGINAL SLEW VELOCITY IS RESTORED AFTER HOMING.
%             %m is an object of EZStepper class
% 
%             %TEST LIMITS
%             [limits,error] = MotorReadLimits(m);
%             if((limits>0)||(error~=0)), return; end
%             %READ CURRENT SLEW VELOCITY
%             [slewvelo,error] = MotorReadSlewVelo(m);
%             if(error~=0), return; end
%             %START MOVE
%             error = MotorJog(m,speed);
%             if(error~=0), MotorJog(m,0); return; end
%             %LOOP AND CHECK
%             tic;
%             while(1)
%                 [limits,error] = MotorReadLimits(m);
%                 if((limits>0)&&(error==0)) %if((limits>0)||(error~=0))
%                     break;
%                 end
%                 if(toc>m.Timeout)
%                     error = -1; break;
%                 end
%             end
%             %STOP MOVE
%             er1 = MotorJog(m,0);
%             %RESTORE SLEW VELOCITY
%             set(m,'SlewVelocity',slewvelo); er2=0; %er2 = MotorSetSlewVelo(m,slewvelo);
%             %RETURN ERROR
%             if(error~=0), return; end
%             if(er1~=0), error = er1; return; end
%             if(er2~=0), error = er2; return; end
%         end
        function [limits,error] = MotorToLimit(m,speed)
            %MOVE UNTIL STOPPED BY LIMIT. RETURNS THE LIMIT THAT WAS HIT.
            %SIGN OF speed DETERMINES DIRECTION OF MOVE.
            %limit: 1-limit 1(A), 2-limit 2(B)
            %ORIGINAL SLEW VELOCITY IS RESTORED AFTER HOMING.
            %m is an object of EZStepper class

            % READ CURRENT SLEW VELOCITY
            [slewvelo,error] = MotorReadSlewVelo(m);
            if(error~=0), limits = NaN; return; end
            % START MOVE
            er1 = MotorJog(m,speed);
            % WAIT FOR COMPLETION
            er2 = MotorWait(m);
            % READ LIMITS
            [limits,er3] = MotorReadLimits(m);
            
            % START MOVE OUT
            er4 = MotorJog(m,-speed/10);
            %LOOP AND CHECK
            tic;
            while(1)
                [limits,~] = MotorReadLimits(m);
                if(limits==0), break; end
                if(toc>m.Timeout), error = 32; break; end
            end
            %STOP MOVE
            er5 = MotorJog(m,0);
            
            %RESTORE SLEW VELOCITY
            set(m,'SlewVelocity',slewvelo); 
            %RETURN ERROR
            if(error~=0), return; end
            error = er1+2*er2+4*er3+8*er4+16*er5;
        end
    end
    methods (Access = private)
        function error = MotorSetDivide(m)
            %SET MICROSTEP DIVIDE FOR EZStepper CONTROLLER
            %MICROSTEP = FULL_STEP/DIVIDE
            %VALID VALUES: 1(FULL STEP), 2, 4, 8, 16, 32, 64, 128, 256.
            %m is an object of EZStepper class
            divide = m.StepDivide;
            switch divide
                case {1, 2, 4, 8, 16, 32, 64, 128, 256}
                    %VALID VALUES
                otherwise
                    %error('Invalid microstep divide');
                    set(m,'LastErr',-1);
                    error = -1;
                    return;
            end
            cmnd = ['j' sprintf('%d',divide) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end      
        function error = MotorSetHoldCurrent(m)
            %SET HOLD CURRENT FOR EZStepper CONTROLLER (IN % OF MAX = 2 AMPS [0-50]).
            %m is an object of EZStepper class
            holdcurrent = m.HoldCurrent;
            holdcurrent = fix(holdcurrent);
            if(holdcurrent<0), holdcurrent = 0; end
            if(holdcurrent>50), holdcurrent = 50; end
            cmnd = ['h' sprintf('%d',holdcurrent) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetRunCurrent(m)
            %SET RUN CURRENT FOR EZStepper CONTROLLER (IN % OF MAX = 2 AMPS, [0-100]).
            %m is an object of EZStepper class
            runcurrent = m.RunCurrent;
            runcurrent = fix(runcurrent);
            if(runcurrent<0), runcurrent = 0; end
            if(runcurrent>100), runcurrent = 100; end
            cmnd = ['m' sprintf('%d',runcurrent) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetInitVelo(m)
            %SET INITIAL VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            % !!! Max allowed ~15000 (in j256)
            %m is an object of EZStepper class
            initvelo = m.InitVelocity;
            ivel = fix(abs(initvelo));
            cmnd = ['v' sprintf('%d',ivel) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetSlewVelo(m)
            %SET SLEW VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            %m is an object of EZStepper class
            slewvelo = m.SlewVelocity;
            svel = fix(abs(slewvelo));
            cmnd = ['V' sprintf('%d',svel) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetRamp(m)
            %SET ACCELERATION FOR EZStepper CONTROLLER (IN MICROSTEPS/SEC^2).
            %DECELERATION IS THE SAME AS ACCELERATION.
            %m is an object of EZStepper class
            acceleration = m.Ramp;
            accel = fix(abs(acceleration));
            cmnd = ['L' sprintf('%d',accel) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetLimits(m)
            %ENABLE (1) OR DISABLE (0) LIMITS FOR EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %DEFINED BY BIT 1 (if counting from 0).
            enable = m.LimitEnable;
            if(enable~=0), enable=1; end
            nb = m.nBits;
            unb = uint8(nb); 
            unb = bitset(unb,2,enable);
            set(m,'nBits',unb);
            cmnd = ['n' sprintf('%d',unb) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
        function error = MotorSetLimPolar(m)
            %SET POLARITY OF LIMIT SWITCHES FOR EZHR17EN CONTROLLER.
            %0-NORMAL(ACTIVE HIGH), 1-INVERTED (ACTIVE LOW)
            %BY DEFAULT ASSUMES THAT SWITCHES ARE NORMALLY CLOSED.
            %m is an object of EZStepper class
            limpol = m.LimitPolarity;
            cmnd = ['f' sprintf('%d',limpol) 'R'];
            [~,error] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',error);
        end
    end
end
function [res, error] = ReadSerialToCRLF(serialobj,timeout)
    % READ RESPONSE UNTIL TERMINATORS 13=CR followed by 10=LF
    % Replaces fgets which does not work 
    % timeout is in seconds
    % return string contains terminators
    cr = 13; lf = 10;
    out = []; 
    tic;
    while(1)
        if(serialobj.BytesAvailable>0)
            out = [out fread(serialobj,serialobj.BytesAvailable,'uint8')'];
            ind = find(out==lf,1,'last');
            if(~isempty(ind)&&(ind>1))
                if(out(ind-1)==cr)
                    %disp(out);
                    res = char(out);
                    error = 0;
                    return;
                end
            end
        end
        elapsed = toc;
        if(elapsed > timeout), res = ''; error = 1; return; end
    end
end
function res = ReadSerialToTimeout(serialobj,timeout)
    % READ RESPONSE UNTIL TIMEOUT
    % timeout is in seconds
    out = []; 
    tic;
    while(1)
        if(serialobj.BytesAvailable>0)
            out = [out fread(serialobj,serialobj.BytesAvailable,'uint8')'];
        end
        elapsed = toc;
        if(elapsed > timeout), res = char(out); return; end
    end
end
function FlushSerial(serialobj)
    % READ WHAT AVAILABLE - CLEAR THE BUFFER
    if(serialobj.BytesAvailable>0)
        fread(serialobj,serialobj.BytesAvailable,'uint8');
    end
end
