classdef XYStage < hgsetget 
    properties (SetAccess = private)
        PortObj; %port object; must be open
        ControllerType = 'EZHR17'; %supported: EZHR17
        ControllerNameX = 3; %name or number of X controller on this port
        ControllerNameY = 4; %name or number of Y controller on this port
        LeadUmX = 1000; % microns per full motor turn
        LeadUmY = 1000; % microns per full motor turn
        MotorStepsX = 200; %physical steps in the motor; typically 200 or 400
        MotorStepsY = 200; %physical steps in the motor; typically 200 or 400
        LimitPolarity = 0; % limit polarity: 0 normally closed, 1 normally open
        WithLimits = 1; %1 - if has limits; 0 - without limits
        EncoderCounts = 1024; %Real encoder counts (encoder is optional)
    end
    properties
        HoldCurrentX = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
        HoldCurrentY = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
        RunCurrentX = 65; %current when moving; in % of full (2A for EZHR17); typically 50%
        RunCurrentY = 65; %current when moving; in % of full (2A for EZHR17); typically 50%
        StepDivideX = 256; %number of microsteps per step; typically 256
        StepDivideY = 256; %number of microsteps per step; typically 256
        SlewVelocityX = 10000; %slew (max) velocity in microns/sec 
        SlewVelocityY = 10000; %slew (max) velocity in microns/sec 
        InitVelocityX = 500; %initial velocity in microns/sec 
        InitVelocityY = 500; %initial velocity in microns/sec 
        RampX = 100; %acceleration (deceleration) in microns/sec^2 
        RampY = 100; %acceleration (deceleration) in microns/sec^2 
        Log = 0; %0 - do not accumulate log; 1 - accumulate log
        LastErrX = 0; %Last error
        LastErrY = 0; %Last error
    end
    properties (SetAccess = private, GetAccess = private)
        ezs_x; %EZStepper object for X
        ezs_y; %EZStepper object for Y
        msm_x; %micron to microstep conversion in X
        msm_y; %micron to microstep conversion in Y
        enc_x; %micron to encoder conversion in X
        enc_y; %micron to encoder conversion in Y
    end
    methods
        function xy = XYStage(varargin)
            property_argin = varargin;
            while (length(property_argin) >= 2)
                prop = property_argin{1};
                val = property_argin{2};
                property_argin = property_argin(3:end);
                switch prop
                case 'PortObj'
                    xy.PortObj = val;
                case 'ControllerType'
                    xy.ControllerType = val;
                case 'ControllerNameX'
                    xy.ControllerNameX = val;
                case 'ControllerNameY'
                    xy.ControllerNameY = val;
                case 'LeadUmX'
                    xy.LeadUmX = val;
                case 'LeadUmY'
                    xy.LeadUmY = val;
                case 'MotorStepsX'
                    xy.MotorStepsX = val;
                case 'MotorStepsY'
                    xy.MotorStepsY = val;
                case 'LimitPolarity'
                    xy.LimitPolarity = val;
                case 'HoldCurrentX'
                    xy.HoldCurrentX = val;
                case 'HoldCurrentY'
                    xy.HoldCurrentY = val;
                case 'RunCurrentX'
                    xy.RunCurrentX = val;
                case 'RunCurrentY'
                    xy.RunCurrentY = val;
                case 'StepDivideX'
                    xy.StepDivideX = val;
                case 'StepDivideY'
                    xy.StepDivideY = val;
                case 'SlewVelocityX'
                    xy.SlewVelocityX = val;
                case 'SlewVelocityY'
                    xy.SlewVelocityY = val;
                case 'InitVelocityX'
                    xy.InitVelocityX = val;
                case 'InitVelocityY'
                    xy.InitVelocityY = val;
                case 'RampX'
                    xy.RampX = val;
                case 'RampY'
                    xy.RampY = val;
                case 'Log'
                    xy.Log = val;
                case 'WithLimits'
                    xy.WithLimits = val;
                otherwise
                    error([prop,' Is not a valid name'])
                end
            end
            % PARAMETER CHECKS
            % RECALCULATION COEFFICIENTS FROM MICRONS TO MICROSTEPS
            MicronToMicrostep(xy);  
            % RECALCULATION COEFFICIENTS FROM MICRONS TO ENCODER COUNTS
            MicronToEncoder(xy);
            % X
            slewms_x = fix(xy.SlewVelocityX*xy.msm_x);
            initms_x = fix(xy.InitVelocityX*xy.msm_x);
            rampms_x = fix(xy.RampX*xy.msm_x);
            %CREATE EZStepper OBJECT FOR X WITH SUPPLIED PARAMETERS
            mx = EZStepper('PortObj',xy.PortObj, 'ControllerName', xy.ControllerNameX, ...
                'Timeout',30, 'LimitEnable',xy.WithLimits, 'StepDivide',xy.StepDivideX, ...
                'SlewVelocity',slewms_x, 'InitVelocity',initms_x, 'Ramp',rampms_x, ...
                'HoldCurrent',xy.HoldCurrentX, 'RunCurrent',xy.RunCurrentX, ...
                'LimitPolarity',xy.LimitPolarity, 'Log',xy.Log); 
            xy.LastErrX = mx.LastErr;
            xy.ezs_x = mx;
            % Y
            slewms_y = fix(xy.SlewVelocityY*xy.msm_y);
            initms_y = fix(xy.InitVelocityY*xy.msm_y);
            rampms_y = fix(xy.RampY*xy.msm_y);
            %CREATE EZStepper OBJECT FOR Y WITH SUPPLIED PARAMETERS
            my = EZStepper('PortObj',xy.PortObj, 'ControllerName', xy.ControllerNameY, ...
                'Timeout',30, 'LimitEnable',xy.WithLimits, 'StepDivide',xy.StepDivideY, ...
                'SlewVelocity',slewms_y, 'InitVelocity',initms_y, 'Ramp',rampms_y, ...
                'HoldCurrent',xy.HoldCurrentY, 'RunCurrent',xy.RunCurrentY, ...
                'LimitPolarity',xy.LimitPolarity, 'Log',xy.Log); 
            xy.LastErrY = my.LastErr;
            xy.ezs_y = my;
        end % xy
        function xy = set.HoldCurrentX(xy,val)
           xy.HoldCurrentX = val; 
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.HoldCurrent = val; end
        end    
        function xy = set.HoldCurrentY(xy,val)
           xy.HoldCurrentY = val; 
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.HoldCurrent = val; end
        end    
        function xy = set.RunCurrentX(xy,val)
           xy.RunCurrentX = val; 
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.RunCurrent = val; end
        end    
        function xy = set.RunCurrentY(xy,val)
           xy.RunCurrentY = val; 
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.RunCurrent = val; end
        end    
        function xy = set.StepDivideX(xy,val)
           xy.StepDivideX = val; 
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.StepDivide = val; end
        end    
        function xy = set.StepDivideY(xy,val)
           xy.StepDivideY = val; 
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.StepDivide = val; end
        end    
        function xy = set.SlewVelocityX(xy,val)
           xy.SlewVelocityX = val; 
           slewms_x = fix(xy.SlewVelocityX*xy.msm_x);
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.SlewVelocity = slewms_x; end
        end    
        function xy = set.SlewVelocityY(xy,val)
           xy.SlewVelocityY = val; 
           slewms_y = fix(xy.SlewVelocityY*xy.msm_y);
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.SlewVelocity = slewms_y; end
        end    
        function xy = set.InitVelocityX(xy,val)
           xy.InitVelocityX = val;
           initms_x = fix(xy.InitVelocityX*xy.msm_x);
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.InitVelocity = initms_x; end
        end    
        function xy = set.InitVelocityY(xy,val)
           xy.InitVelocityY = val;
           initms_y = fix(xy.InitVelocityY*xy.msm_y);
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.InitVelocity = initms_y; end
        end    
        function xy = set.RampX(xy,val)
           xy.RampX = val; 
           rampms_x = fix(xy.RampX*xy.msm_x);
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.Ramp = rampms_x; end
        end   
        function xy = set.RampY(xy,val)
           xy.RampY = val; 
           rampms_y = fix(xy.RampY*xy.msm_y);
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.Ramp = rampms_y; end
        end   
        function xy = set.Log(xy,val)
           xy.Log = val; 
           mx = xy.ezs_x; 
           if(strcmp(class(mx),'EZStepper')), mx.Log = val; end
           my = xy.ezs_y; 
           if(strcmp(class(my),'EZStepper')), my.Log = val; end
        end  
        function delete(xy)
           mx = xy.ezs_x; delete(mx);
           my = xy.ezs_y; delete(my);
        end
        function err = XYRelative(xy,travelx,travely)
            travstepsx = fix(travelx*xy.msm_x);
            travstepsy = fix(travely*xy.msm_y);
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx = MotorMoveRel(mx,travstepsx);
            erry = MotorMoveRel(my,travstepsy);
            err = errx+erry;
        end
        function err = XYAbsolute(xy,destinationx,destinationy)
            deststepsx = fix(destinationx*xy.msm_x);
            deststepsy = fix(destinationy*xy.msm_y);
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx = MotorMoveAbs(mx,deststepsx);
            erry = MotorMoveAbs(my,deststepsy);
            err = errx+erry;
        end
        function err = XYJog(xy, speedx,speedy)
            stepspeedx = fix(speedx*xy.msm_x);
            stepspeedy = fix(speedy*xy.msm_y);
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx = MotorJog(mx,stepspeedx);
            erry = MotorJog(my,stepspeedy);
            err = errx+erry;
        end
        function err = XYStop(xy)
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx = MotorJog(mx,0);
            erry = MotorJog(my,0);
            err = errx+erry;
        end
        function err = XYHome(xy, homespeedx,homespeedy)
            if(xy.WithLimits==0), return; end
            if(nargin==3)
                hspeedx = homespeedx; hspeedy = homespeedy;
            else
                hspeedx = xy.SlewVelocityX; hspeedy = xy.SlewVelocityY;
            end
            hstepspeedx = fix(hspeedx*xy.msm_x);
            hstepspeedy = fix(hspeedy*xy.msm_y);
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx  = MotorHome(mx,hstepspeedx);
            erry  = MotorHome(my,hstepspeedy);
            err = errx+erry;
        end
        function err = XYHomeComplete(xy, homespeedx,homespeedy)
            if(xy.WithLimits==0), return; end
            if(nargin==3)
                hspeedx = homespeedx; hspeedy = homespeedy;
            else
                hspeedx = xy.SlewVelocityX; hspeedy = xy.SlewVelocityY;
            end
            hstepspeedx = fix(hspeedx*xy.msm_x);
            hstepspeedy = fix(hspeedy*xy.msm_y);
            mx = xy.ezs_x;
            my = xy.ezs_y;
            slewvelox = xy.SlewVelocityX;
            slewveloy = xy.SlewVelocityY;
            %START HOMING
            err2x = MotorHome(mx,hstepspeedx);
            err2y = MotorHome(my,hstepspeedy);
            %WAIT FOR COMPLETION
            err3x = MotorWait(mx);
            err3y = MotorWait(my);
            %RESTORE SLEW VELOCITY
            if(hspeedx~=slewvelox), set(xy,'SlewVelocityX',slewvelox); end
            if(hspeedy~=slewveloy), set(xy,'SlewVelocityY',slewveloy); end
            err = err2x+err2y+err3x+err3y;
        end
        function err = XYHome2Phase(xy, homespeedx,homespeedy)
            if(xy.WithLimits==0), return; end
            if(nargin==3)
                hspeedx = homespeedx; hspeedy = homespeedy;
            else
                hspeedx = xy.SlewVelocityX; hspeedy = xy.SlewVelocityY;
            end
            % PHASE 1 - AT GIVEN SPEED
            err = XYHomeComplete(xy, hspeedx,hspeedy);
            if(err~=0), return; end
            % MOVE OUT
            moveout = 500; % 0.5 mm
            err = XYRelative(xy,moveout,moveout);
            err = XYWait(xy);
            if(err~=0), return; end
            % PHASE 2 - AT SLOW SPEED
            slowfactor = 20;
            err = XYHomeComplete(xy, hspeedx/slowfactor,hspeedy/slowfactor);
        end
        function err = XYHomeToHardStop(xy, x_full,x_back, y_full,y_back, x_lowc,y_lowc, x_spd, y_spd)
            % E.G. err = XYHomeToHardStop(xy,-80000,1000,-55000,1000);
            % SET LOW SPEED
            if(nargin>=9)
                lowslewX = x_spd; lowslewY = y_spd;
            else
                lowslewX = 5000; lowslewY = 5000;
            end
            oldSlewVelocityX = xy.SlewVelocityX; oldSlewVelocityY = xy.SlewVelocityY; 
            xy.SlewVelocityX = lowslewX; xy.SlewVelocityY = lowslewY;
            % SET LOW RUN CURRENT FOR SOFTER STALL
            if(nargin>=7)
                lowruncurrentX = x_lowc; lowruncurrentY = y_lowc; 
            else
                lowruncurrentX = 5; lowruncurrentY = 5;
            end
            oldRunCurrentX = xy.RunCurrentX; oldRunCurrentY = xy.RunCurrentY;
            xy.RunCurrentX = lowruncurrentX; xy.RunCurrentY = lowruncurrentY;
            % MOVE MAX TRAVEL DISTANCE (x_full, y_full)
            err1 = XYRelative(xy,x_full,y_full);
            err2 = XYWait(xy);
            % BACK OUT
            err3 = XYRelative(xy,x_back,y_back);
            err4 = XYWait(xy);
            % SET POSITION
            err5 = XYOrigin(xy);
            % RESTORE OLD PARAMETERS
            xy.RunCurrentX = oldRunCurrentX; xy.RunCurrentY = oldRunCurrentY;
            xy.SlewVelocityX = oldSlewVelocityX; xy.SlewVelocityY = oldSlewVelocityY;
            err = err1+err2*10+err3*100+err4*1000+err5*10000;
        end
        function err = XYHomeToEncoder(xy, x_full,y_full, x_homespeed,y_homespeed, x_lowc,y_lowc, x_back,y_back, enc_err)
            % Home until encoder stops changing
            % x_full,y_full - max move, usually negative
            % x_homespeed,y_homespeed - speed of move
            % x_lowc,y_lowc - move current for home, usually low because some stall is possible
            % enc_err - encoder error for detection of stall
            % E.G. err = XYHomeToEncoder(xy, -80000,-55000, 5000,5000, 10,10, 1000,1000, 2);
            % SET ENCODER TOLERANCE
            if(nargin==10)
                eps = enc_err;
            else
                eps = 2;
            end
            % SET BACK-UP DISTANCE
            if(nargin>=9)
                backX = x_back; backY = y_back;
            else
                backX = 1000; backY = 1000;
            end
            % SET LOW RUN CURRENT FOR SOFTER STALL
            if(nargin>=7)
                lowruncurrentX = x_lowc; lowruncurrentY = y_lowc; 
            else
                lowruncurrentX = 5; lowruncurrentY = 5;
            end
            oldRunCurrentX = xy.RunCurrentX; oldRunCurrentY = xy.RunCurrentY;
            xy.RunCurrentX = lowruncurrentX; xy.RunCurrentY = lowruncurrentY;
            % SET LOW SPEED
            if(nargin>=5)
                lowslewX = x_homespeed; lowslewY = y_homespeed;
            else
                lowslewX = 5000; lowslewY = 5000;
            end
            oldSlewVelocityX = xy.SlewVelocityX; oldSlewVelocityY = xy.SlewVelocityY; 
            xy.SlewVelocityX = lowslewX; xy.SlewVelocityY = lowslewY;
            % START MOVE MAX TRAVEL DISTANCE (x_full, y_full)
            err1 = XYRelative(xy,x_full,y_full);
            %LOOP CHECKING ENCODER
            oldenposx = 0; oldenposy = 0; checkx = 1; checky = 1;
            mx = xy.ezs_x; my = xy.ezs_y;
            while(1)
                if(checkx == 1)
                    [enposx,err2x] = MotorReadEncoder(mx);
                    if(abs(oldenposx-enposx) <= eps) 
                        MotorJog(mx,0); checkx = 0;
                    end
                    oldenposx = enposx;
                end
                if(checky == 1)
                    [enposy,err2y] = MotorReadEncoder(my);
                    if(abs(oldenposy-enposy) <= eps) 
                        MotorJog(my,0); checky = 0;
                    end
                    oldenposy = enposy;
                end
                if((checkx == 0) && (checky == 0)), break; end
            end
            % SETTLE AFTER STALL
            pause(0.5);
            % RESTORE OLD PARAMETERS
            xy.RunCurrentX = oldRunCurrentX; xy.RunCurrentY = oldRunCurrentY;
            xy.SlewVelocityX = oldSlewVelocityX; xy.SlewVelocityY = oldSlewVelocityY;
            % BACK OUT
            err3 = XYRelative(xy, backX, backY);
            err4 = XYWait(xy);
            % SET POSITION
            err5 = XYOrigin(xy);
            err = err1+err2x*10+err2y*100+err3*1000+err4*10000+err5*100000;
        end
        function err = XYWait(xy)
            mx = xy.ezs_x;
            my = xy.ezs_y;
            errx = MotorWait(mx);
            erry = MotorWait(my);
            err = errx+erry;
        end
        function err = XYOrigin(xy, originx,originy)
            if(nargin == 3)
                origx = originx; origy = originy;
            else
                origx = 0; origy = 0;
            end
            mx = xy.ezs_x;
            my = xy.ezs_y;
            origmsx = fix(origx*xy.msm_x);
            origmsy = fix(origy*xy.msm_y);
            errx = MotorSetOrigin(mx,origmsx);
            erry = MotorSetOrigin(my,origmsy);
            err = errx+erry;
        end
        function [posx, posy, err] = XYPos(xy)
            mx = xy.ezs_x;
            my = xy.ezs_y;
            [positionx,errx] = MotorReadPosition(mx);
            [positiony,erry] = MotorReadPosition(my);
            posx = positionx/xy.msm_x;
            posy = positiony/xy.msm_y;
            err = errx+erry;
        end
        function [enposx, enposy, err] = XYEncoderPos(xy)
            mx = xy.ezs_x;
            my = xy.ezs_y;
            [enc_posx,errx] = MotorReadEncoder(mx);
            [enc_posy,erry] = MotorReadEncoder(my);
            enposx = enc_posx/xy.enc_x;
            enposy = enc_posy/xy.enc_y;
            err = errx+erry;
        end
        function [resp, err] = XYExecuteX(xy,cmd)
            mx = xy.ezs_x;
            [resp,err] = MotorCmndRspn(mx,cmd);
        end
        function [resp, err] = XYExecuteY(xy,cmd)
            my = xy.ezs_y;
            [resp,err] = MotorCmndRspn(my,cmd);
        end
        function err = XYOutX(xy,out)
            mx = xy.ezs_x;
            err = MotorOutputs(mx,out); %out: driver1 + 2*driver2 (driver = 0/1)
        end
        function err = XYOutY(xy,out)
            my = xy.ezs_y;
            err = MotorOutputs(my,out); %out: driver1 + 2*driver2 (driver = 0/1)
        end
        function [limits,err] = XYToLimitX(xy,speed)
            if(nargin==2), spd = speed; else spd = xy.SlewVelocityX; end
            stepspd = fix(spd*xy.msm_x);
            mx = xy.ezs_x;
            [limits,err] = MotorToLimit(mx,stepspd);
        end
        function [limits,err] = XYToLimitY(xy,speed)
            if(nargin==2), spd = speed; else spd = xy.SlewVelocityY; end
            stepspd = fix(spd*xy.msm_y);
            my = xy.ezs_y;
            %Move until any limit turns on. returns the limit that was hit.
            %if already at the limit, returns immediately. sign of speed determines direction of move.
            %limit: 1-limit 1(a), 2-limit 2(b); original slew velocity is restored after homing.
            [limits,err] = MotorToLimit(my,stepspd);
        end
    end
    methods (Access = private)
        function MicronToMicrostep(xy)
            %MICRON TO MICROSTEP CONVERSION: msm - number of microsteps in 1 micron
            xy.msm_x = xy.MotorStepsX*xy.StepDivideX/xy.LeadUmX;    
            xy.msm_y = xy.MotorStepsY*xy.StepDivideY/xy.LeadUmY;    
        end
        function MicronToEncoder(xy)
            %MICRON TO ENCODER CONVERSION: enc - number of quadrature encoder counts in 1 micron
            xy.enc_x = xy.EncoderCounts*4/xy.LeadUmX;    
            xy.enc_y = xy.EncoderCounts*4/xy.LeadUmY;    
        end
    end    
end

